## About
Schema is an open-source component of [Memri](https://blog.memri.io). It functions as the single source of truth of the
data model of your personal data.

This repository contains code to export schema definitions for
[Pod](https://gitlab.memri.io/memri/pod),
[iOS app](https://gitlab.memri.io/memri/ios-application)
and [Flutter/Android/Desktop/Web app](https://gitlab.memri.io/memri/flutter-app).

You may need to interact with this repository
if you need to [change the schema](#Changing the schema).

## Local build/install
To run the code, make sure you have [Node.js and npm](https://www.npmjs.com/get-npm) installed and run:
```bash
cd tools/
```

To test whether the schema contains inconsistencies: `node ../tests/test.js`  
To export the schema json for the iOS and Flutter application: `node export_frontend_schema.js`  

## The schema
The schema consists of `types`, which is a set of files to define items types and characteristics. `types` defines:
* What types of Items exist
* What properties and edges an Item can have
* Types of Edges and Properties of Item
* Inheritance of Items

### Items
All the data points in Memri are called items, e.g. a Person, a Location or a Video. The top level item is called Item, 
and all other items inherit all the properties and edges of Item.

#### Edges
Edges are connections to other items, for instance, there might be an 'attends' edge from a Person to an Event.

#### Properties
Where edges connect to other items, properties connect directly to a value of a primitive type, for instance, an item 
could have a 'name' property of type 'String'. Properties are always one-to-one relations: An item can only have single 
value for a property.

### Primitive types
The primitive types that are supported by the data model are:
* "Text": A string
* "Integer": An integer
* "Real": A float
* "Bool": True or False
* "DateTime": DateTime in Unix format

All properties must expect one of these types. 

## Changing the schema
The schema is stored in the directory `types` as follows.
* The name of the json file is the name of the item
* The json file specifies the actual item:
```json
{
  "description": "A short description of the item.",
  "base_schema": "SomeItem",
  "properties": [
       {"name": "someProperty", "type": "SomeType"}
  ],
  "edges": [
    {"name": "someEdge", "target": "SomeItem"}
  ]
}
```
Where:
* `base_schema` is other item from which current item inherits properties and edges. If not set, will be considered as `Item`
* property `name` is the name of the property and `type` is one of `Primitive types`
* edge `name` is the name of the edge and `target` should point to one of existing items or to `Any` 

If you added an Item, Edge or Property, make sure to run `node ../tests/test.js` to see whether there are inconsistencies.
 
If there aren't, use export described earlier to create the schemas for the projects you are working on,
 and overwrite them in the corresponding projects and rebuild those. We plan to automate this process further in the
 near future to improve the developer experience.