const helpers = require('../tools/helpers');
const path = require('path');

const typesPath = path.resolve('../types');


(async () => {
  let types = await helpers.getTypes(typesPath);

  let allPropertiesTypes = {};
  let allEdgesTypes = {};

  let differentPropertiesTypes = [];
  let differentEdgesTypes = [];

  console.log('\nCheck if Types of Properties are valid...');
  for (const item of Object.keys(types)) {
    for (const property of types[item]["properties"]) {
      if (property["name"] === undefined) {
        console.log(`E: Item: '${item}', has property with no name.`);
      } else if (property["type"] === undefined) {
        console.log(`E: Item: '${item}', property: '${property["name"]}' has no type.`);
      } else if (!helpers.PRIMITIVE_TYPES.includes(property["type"])) {
        console.log(`E: Item: '${item}', property: '${property["name"]}' expects non-existent type: '${property["type"]}'.`);
      }

      if (property["name"] === "type") {
        console.log(`E: Item: '${item}', has property 'type' which is deprecated as is used in pod as item type.`);
      }

      if (allPropertiesTypes[property["name"]] !== undefined && allPropertiesTypes[property["name"]] !== property["type"]) {
        differentPropertiesTypes.push({...property, ...{"item": item}});
      } else {
        allPropertiesTypes[property["name"]] = property["type"];
      }
    }
  }

  console.log('\nCheck if Types of Edges are valid...');
  for (const item of Object.keys(types)) {
    for (const edge of types[item]["edges"]) {
      if (edge["name"] === undefined) {
        console.log(`E: Item: '${item}', has edge with no name.`);
      } else if (edge["target"] === undefined) {
        console.log(`E: Item: '${item}', edge: '${edge["name"]}' has no target.`);
      } else if (!Object.keys(types).includes(edge["target"]) && edge["target"] !== "Any") {
        console.log(`E: Item: '${item}', edge: '${edge["name"]}' expects non-existent type: '${edge["target"]}'.`);
      }
      if (allEdgesTypes[edge["name"]] !== undefined && allEdgesTypes[edge["name"]] !== edge["target"]) {
        differentEdgesTypes.push({...edge, ...{"item": item}});
      } else {
        allEdgesTypes[edge["name"]] = edge["target"];
      }
    }
  }

  console.log('\nCheck for different Properties types...');
  if (differentPropertiesTypes.length > 0) {
    for (const differentProperty of differentPropertiesTypes) {
      console.log(`E: Item: '${differentProperty["item"]}', has different property: '${differentProperty["name"]}' type: '${differentProperty["type"]}',  than declined before'.`);
    }
  }

  console.log('\nCheck for different Edges types...');
  if (differentEdgesTypes.length > 0) {
    for (const differentEdge of differentEdgesTypes) {
      console.log(`E: Item: '${differentEdge["item"]}', has different edge: '${differentEdge["name"]}' target: '${differentEdge["target"]}',  than declined before'.`);
    }
  }




  let errorParentList = [];

  console.log('\nCheck if inheritances are valid...');
  for (const item of Object.keys(types)) {
    if (types[item]["base_schema"] === undefined) {
      continue;
    }
    if (!Object.keys(types).includes(types[item]["base_schema"])) {
      console.log(`E: Item: '${item}', inherits from unknown item: '${types[item]["base_schema"]}'.`);
    }

    let parentItem = item;
    let parentItemList = [];

    while (parentItem) {
      if (parentItemList.includes(parentItem)) {
        console.log(`E: Item: '${parentItem}', inherits itself recursively at some point star.`);
        errorParentList.push(parentItem);
        break;
      }
      parentItemList.push(parentItem);
      parentItem = types[parentItem]["base_schema"];
    }
  }

  console.log('\nCheck for duplicate properties and edges (duplicated or inherited and redefined)...');
  for (const item of Object.keys(types)) {
    let usedProperties = []; let usedEdges = [];

    for (const property of types[item]["properties"]) {
      if (usedProperties.includes(property["name"])) {
        console.log(`E: item ${item} has duplicate property '${property["name"]}'.`);
      }
      usedProperties.push(property["name"]);
    }

    for (const edge of types[item]["edges"]) {
      if (usedEdges.includes(edge["name"])) {
        console.log(`E: item ${item} has duplicate edge '${edge["name"]}'.`);
      }
      usedEdges.push(edge["name"]);
    }

    if (types[item]["base_schema"] === undefined) {
      continue;
    }

    let parentItem = types[item]["base_schema"];

    while (parentItem) {
      if (errorParentList.includes(parentItem)) {
        break;
      }
      for (const property of types[parentItem]["properties"]) {
        if (usedProperties.includes(property["name"])) {
          console.log(`E: item ${item} redefines property '${property["name"]}' that is already in ${parentItem}.`);
        }
      }

      for (const edge of types[parentItem]["edges"]) {
        if (usedEdges.includes(edge["name"])) {
          console.log(`E: item ${item} redefines edge '${edge["name"]}' that is already in ${parentItem}.`);
        }
      }

      parentItem = types[parentItem]["base_schema"];
    }
  }

  console.log('\nCheck for problems in descriptions...');
  for (const item of Object.keys(types)) {
    if (types[item]['description']) {
      if (types[item]['description'].toLowerCase().includes('tbd')) {
        console.log(`W: Item '${item}' has TBD in description.`);
      }
    } else {
      console.log(`W: Item '${item}' is missing a description.`);
    }
  }
})();