import importlib
import inspect
import os
import subprocess
from pathlib import Path

import pytest
from pymemri.pod.client import PodClient


@pytest.fixture(autouse=True, scope="session")
def setup_cleanup():
    os.chdir(Path(__file__).parent)
    yield None
    Path("./central_schema.py").unlink(missing_ok=True)


def test_export_pymemri():
    subprocess.call(
        ["python", "../tools/export_pymemri.py", "-o", "./central_schema.py"]
    )


def test_add_to_schema():
    central_schema = importlib.import_module("central_schema")
    members = inspect.getmembers(central_schema)
    schema_classes = [
        cls
        for _, cls in members
        if inspect.isclass(cls) and cls.__module__ == "central_schema"
    ]

    client = PodClient()

    # NOTE Bulk add_to_schema will not throw errors with inconsistent property types,
    # adding 1 by 1 to test for inconsistent types.
    # See https://gitlab.memri.io/memri/pod/-/issues/408
    for cls in schema_classes:
        assert client.add_to_schema(cls), f"Could not add {cls.__name__}"
