const fs = require('fs');
const { readdir } = require('fs').promises;
const helpers = require('./helpers');
const path = require('path');

const typesPath = path.resolve('../types');
const outputFile = '../auto-gen/schema.json';

function getItemData(item, itemList) {
    let itemProperties = [...itemsJsons[item]["properties"]];
    let itemEdges = [...itemsJsons[item]["edges"]];

    itemList = itemList || [];
    if (itemList.includes(item)) {//check for recursive inheritance
        return [itemProperties, itemEdges];
    }
    itemList.push(item);

    if (itemsJsons[item]["base_schema"] != null) {
        let [parentProperties, parentEdges] = getItemData(itemsJsons[item]["base_schema"], itemList);
        itemProperties.push(...parentProperties);
        itemEdges.push(...parentEdges);
    }

    return [itemProperties, itemEdges];
}

function getDataItemPropertiesAndEdges() {
    let schemaItemsProperties = [];
    let schemaItemEdges = [];

    for (const item of Object.keys(itemsJsons).sort()) {
        let [itemProperties, itemEdges] = getItemData(item);
        let usedProperties = []; let usedEdges = [];

        for (const property of itemProperties
            .filter((value) => !usedProperties.includes(value["name"]) && usedProperties.push(value["name"]))
            .sort((a, b) => a["name"] > b["name"] ? 1 : -1)) {
            if (property["name"] === "type") {
                console.log(`E: Item: '${item}', has property 'type' which is deprecated as is used in pod as item type.`);
                throw Error("Could not export schema");
            }
            schemaItemsProperties.push(`    {`,
                `      "type": "ItemPropertySchema",`,
                `      "itemType": "${item}",`,
                `      "propertyName": "${property["name"]}",`,
                `      "valueType": "${property["type"]}"`,
                `    },`);
        }

        if (usedProperties.length !== itemProperties.length) {
            console.log(item + " contains duplicate properties");
        }


        for (const edge of itemEdges
            .filter((value) => !usedEdges.includes(value["name"]) && usedEdges.push(value["name"]))
            .sort((a, b) => a["name"] > b["name"] ? 1 : -1)) {
            schemaItemEdges.push(`    {`,
                `      "type": "ItemEdgeSchema",`,
                `      "sourceType": "${item}",`,
                `      "targetType": "${edge["target"]}",`,
                `      "edgeName": "${edge["name"]}"`,
                `    },`);
        }

        if (usedEdges.length !== itemEdges.length) {
            console.log(item + " contains duplicate edges");
        }
    }

    schemaItemEdges[schemaItemEdges.length - 1] = schemaItemEdges[schemaItemEdges.length - 1].slice(0, -1);

    return schemaItemsProperties.concat(schemaItemEdges);
}

let itemsJsons = {};
(async () => {
    itemsJsons = await helpers.getTypes(typesPath);

    const schemaItems = getDataItemPropertiesAndEdges();

    const output = `[
${helpers.insertList(schemaItems, 0)}
]`;

    fs.writeFile(outputFile, output, (err) => {
        if (err) throw err;
        console.log('File saved as ' + outputFile);
    });
})();