const fs = require('fs');
const {readdir} = require('fs').promises;

const PRIMITIVE_TYPES = ["Text", "Integer", "Real", "Bool", "DateTime"];

function wrapText(str, width, spaceReplacer) {
  spaceReplacer = spaceReplacer || '\n    /// ';
  if (str.length > width) {
    let p = width;
    for (; p > 0 && str[p] !== ' '; p--) {
    }
    if (p > 0) {
      let left = str.substring(0, p);
      let right = str.substring(p + 1);
      return left + spaceReplacer + wrapText(right, width, spaceReplacer);
    }
  }
  return str;
}

async function getTypes(dir) {
  let types = {};
  const dirents = await readdir(dir, {withFileTypes: true});
  for (const dirent of dirents) {
    let data = fs.readFileSync(`${dir}/${dirent.name}`);
    types[dirent.name.split('.')[0]] = JSON.parse(data);
  }

  return types;
}

function insertList(content, indent) {
  let output = '';
  for (const [i, line] of content.entries()) {
    if (i !== 0) output += ' '.repeat(indent);
    output += line;
    if (i !== (content.length - 1)) output += '\n';
  }
  return output;
}

module.exports = {
  wrapText,
  getTypes,
  insertList,
  PRIMITIVE_TYPES
};
